local _,PlayerClass = UnitClass("player")

local mediapath = "Interface\\AddOns\\v6_oUF\\media\\"

local AuraBackdrop = {
  bgFile = nil,
  edgeFile = mediapath.."aura_border.tga", tile = false, tileSize = 0, edgeSize = 3,
  insets = { left = 0, right = 0, top = 0, bottom = 0 }
}

local FAKE = function() return "FAKE" end

local SetBarWidth = function(self, decimal)
	if decimal <= 0 then
		self:Hide()
	else
		self:Show()
		self:SetWidth(self.width*decimal)
		self:SetTexCoord(math.abs(decimal-1), 1, 0, 1)
	end
end

local siValue = function(val)
	if(val >= 1e6) then
		return ("%.1fm"):format(val / 1e6)
	elseif(val >= 1e3) then
		return ("%.1fk"):format(val / 1e3)
	else
		return val
	end
end

local menu = function(self)
	local unit = self.unit:sub(1, -2)
	local cunit = self.unit:gsub("(.)", string.upper, 1)
	if(unit == "party" or unit == "partypet") then
		ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor", 0, 0)
	elseif(_G[cunit.."FrameDropDown"]) then
		ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor", 0, 0)
	end
end

local HealthUpdate = function(self, event, unit)
	if(self.unit ~= unit) then return end
	local health = self.Health
	if(health.PreUpdate) then health:PreUpdate(unit) end
	local current, maximum = UnitHealth(unit), UnitHealthMax(unit)
	local disconnected = not UnitIsConnected(unit)
	local dec = maximum == 0 and 0 or current/maximum
	SetBarWidth(health.bar, dec)
	health.disconnected = disconnected
	health.unit = unit
	if(health.PostUpdate) then return health:PostUpdate(unit, current, maximum) end
end

local PostHealthUpdate = function(self, unit, current, maximum)
	if not UnitIsConnected(unit) then
		self.text:SetText("Disconncted")
	elseif UnitIsDead(unit) or UnitIsGhost(unit) then
		self.text:SetText("Dead")
	else
		local p = floor( current/maximum * 100 + .5)
		current = siValue(current)
		self.text:SetText(p.."% |cFF666666".. current .."|r")
	end
	if unit == "player" then return end
	local name = UnitName(unit)
	if unit == "target" then
		local lv = UnitLevel(unit)
		if lv>0 then
			name = lv.." "..name
		end
		local cl = UnitClassification(unit)
		if cl=="worldboss" then
			name = "Boss "..name
		end
	end
	self:GetParent().Name:SetText(name)
end

local PowerUpdate = function(self, event, unit)
	if(self.unit ~= unit) then return end
	local power = self.Power
	if(power.PreUpdate) then power:PreUpdate(unit) end
	local current, maximum = UnitPower(unit), UnitPowerMax(unit)
	local disconnected = not UnitIsConnected(unit)
	local dec = maximum == 0 and 0 or current/maximum
	SetBarWidth(power.bar, dec)
	power.disconnected = disconnected
	power.unit = unit
	if(power.PostUpdate) then return power:PostUpdate(unit, current, maximum) end
end

local PostPowerUpdate = function(self, unit, current, maximum)

	if not UnitIsConnected(unit) or UnitIsDead(unit) or UnitIsGhost(unit) or current==0 then
		self.text:SetText("")
	else
		local current = siValue(current)
		self.text:SetText(current)
	end

	local r, g, b, t
	if UnitIsPlayer(unit) then
		local _, class = UnitClass(unit)
		t = oUF.colors.class[class]
	elseif UnitReaction(unit, 'player') then
		t = oUF.colors.reaction[UnitReaction(unit, "player")]
	end
	if(t) then
		r, g, b = t[1], t[2], t[3]
	end
	if(b) then
		r=r*.8
		g=g*.8
		b=b*.8
		self.bar:SetVertexColor(r, g, b)
		local cb = self:GetParent().Castbar
		if cb then cb:SetStatusBarColor(r, g, b) end
	end
end

local PostCreateIcon = function(self, button)
	button.icon:SetTexCoord(.1, .9, .1, .9)
	button.icon:SetPoint("TOPLEFT", button, "TOPLEFT", 3, -3)
	button.icon:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -3, 3)
	button.icon:SetDrawLayer("OVERLAY")
	button:SetBackdrop(AuraBackdrop)
	button:SetBackdropBorderColor(.2,.2,.2)
	button.cd:SetReverse()
	button.cd:SetAllPoints(button.icon)
	button.cd:SetDrawEdge(true)
	button.count:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -1, 1)
end

local PostUpdateIcon = function(self, unit, icon, index, offset)
	if icon.debuff then
		local dtype = select(5, UnitAura(unit, index, icon.filter))
		local color = DebuffTypeColor[dtype] or DebuffTypeColor.none
		icon:SetBackdropBorderColor(color.r, color.g, color.b)
	else
		icon:SetBackdropBorderColor(.2,.2,.2)
		--[[
		local _,_,_,_,_,_,_,_,stealable = UnitAura(unit,index,icon.filter)
		if PlayerClass=="MAGE" and stealable then
			icon:SetBackdropBorderColor(1,.9,0)
		end
		--]]
	end
end

local SetPosition = function(icons, x)
	if(icons and x > 0) then
		local col = 0
		local row = 0
		local gap = icons.gap
		local sizex = (icons.size or 16) + (icons['spacing-x'] or icons.spacing or 0)
		local sizey = (icons.size or 16) + (icons['spacing-y'] or icons.spacing or 0)
		local anchor = icons.initialAnchor or "BOTTOMLEFT"
		local growthx = (icons["growth-x"] == "LEFT" and -1) or 1
		local growthy = (icons["growth-y"] == "DOWN" and -1) or 1
		local cols = math.floor(icons:GetWidth() / sizex + .5)
		local rows = math.floor(icons:GetHeight() / sizey + .5)

		for i = 1, #icons do
			local button = icons[i]
			if(button and button:IsShown()) then
				if(gap and button.debuff) then
					if(col > 0) then
						col = 0
						row = row + 1
					end
					gap = false
				end

				if(col >= cols) then
					col = 0
					row = row + 1
				end
				button:ClearAllPoints()
				button:SetPoint(anchor, icons, anchor, col * sizex * growthx, row * sizey * growthy)

				col = col + 1
			elseif(not button) then
				break
			end
		end
	end
end

local UnitAuraFilterChange = function(self, unit)
	if UnitCanAttack("player", unit) then
		self.debuffFilter = "HARMFUL|PLAYER"
		self.buffFilter = "HELPFUL"
	else
		self.debuffFilter = "HARMFUL"
		self.buffFilter = "HELPFUL|PLAYER"
	end
end

local CustomCastbarTimeText = function(self, duration)
	if self.casting then
		self.Time:SetFormattedText("%.1f", self.max - duration)
	elseif self.channeling then
		self.Time:SetFormattedText("%.1f", duration)
	end
end

local StyleFunc = function(self, unit)


	self:SetAttribute("initial-height", 56)
	self:SetAttribute("initial-width", 258)
	self:SetAttribute("initial-scale", 1)
	self.menu = menu
	self:RegisterForClicks("AnyUp")
	self:SetAttribute("*type2", "menu")
	self:SetScript("OnEnter", UnitFrame_OnEnter)
	self:SetScript("OnLeave", UnitFrame_OnLeave)


	local hp = CreateFrame("Frame", nil, self)
	hp:SetPoint("BOTTOM", 0, 0)
	hp:SetPoint("LEFT", 2, 0)
	hp:SetWidth(256)
	hp:SetHeight(32)

	hp.bar = hp:CreateTexture(nil, "BORDER")
	hp.bar:SetTexture(mediapath.."bl_statusbar")
	hp.bar:SetPoint("BOTTOM", 0, 0)
	hp.bar:SetPoint("LEFT", 0, 0)
	hp.bar:SetWidth(hp:GetWidth())
	hp.bar:SetHeight(hp:GetHeight())
	--hp:SetVertexColor(.9,.3,.3)
	hp.bar:SetVertexColor(.2,.2,.2)
	hp.bar.width = 256

	hp.start = hp:CreateTexture(nil, "BORDER")
	hp.start:SetTexture(0,0,0)
	hp.start:SetWidth(2)
	hp.start:SetHeight(hp:GetHeight())
	hp.start:SetPoint("RIGHT", hp, "LEFT", 0, 0)

	hp.bg = hp:CreateTexture(nil, "BACKGROUND")
	hp.bg:SetTexture(mediapath.."bl_statusbar_bg")
	hp.bg:SetVertexColor(.7,.7,.7,.6)
	hp.bg:SetPoint("BOTTOM", 0, 0)
	hp.bg:SetPoint("LEFT", 0, 0)
	hp.bg:SetWidth(hp:GetWidth())
	hp.bg:SetHeight(hp:GetHeight())

	hp.text = hp:CreateFontString(nil, "OVERLAY")
	hp.text:SetFont(mediapath.."kimberley.ttf", 24, "THICKOUTLINE")
	hp.text:SetPoint("LEFT", hp.bg, "LEFT", 2, -2)
	hp.text:SetJustifyH("LEFT")

	hp.frequentUpdates = true
	hp.GetStatusBarTexture = FAKE
	hp.Update = HealthUpdate
	hp.PostUpdate = PostHealthUpdate
	self.Health = hp


	local mp = CreateFrame("Frame", nil, self)
	mp:SetPoint("TOP", 0, 0)
	mp:SetPoint("LEFT", 2, 0)
	mp:SetWidth(170)
	mp:SetHeight(21)

	mp.bar = mp:CreateTexture(nil, "BORDER")
	mp.bar:SetTexture(mediapath.."bl_statusbar2")
	mp.bar:SetPoint("TOP", 0, 0)
	mp.bar:SetPoint("LEFT", 0, 0)
	mp.bar:SetWidth(mp:GetWidth())
	mp.bar:SetHeight(mp:GetHeight())
	--mp:SetVertexColor(.3,.7,.8)
	mp.bar:SetVertexColor(.2,.2,.2)
	mp.bar.width = 170

	mp.start = mp:CreateTexture(nil, "BORDER")
	mp.start:SetTexture(0,0,0)
	mp.start:SetWidth(2)
	mp.start:SetHeight(mp:GetHeight())
	mp.start:SetPoint("RIGHT", mp, "LEFT", 0, 0)

	mp.bg = mp:CreateTexture(nil, "BACKGROUND")
	mp.bg:SetTexture(mediapath.."bl_statusbar_bg")
	mp.bg:SetVertexColor(.7,.7,.7,.6)
	mp.bg:SetPoint("TOP", 0, 0)
	mp.bg:SetPoint("LEFT", 0, 0)
	mp.bg:SetWidth(mp:GetWidth())
	mp.bg:SetHeight(mp:GetHeight())

	mp.text = mp:CreateFontString(nil, "OVERLAY")
	mp.text:SetFont(mediapath.."kimberley.ttf", 12, "THICKOUTLINE")
	mp.text:SetPoint("LEFT", mp.bg, "LEFT", 2, -1)
	mp.text:SetJustifyH("LEFT")

	mp.frequentUpdates = true
	mp.GetStatusBarTexture = FAKE
	mp.Update = PowerUpdate
	mp.PostUpdate = PostPowerUpdate
	self.Power = mp


	local name = self:CreateFontString(nil, "BACKGROUND")
	name:SetFont(mediapath.."kimberley.ttf", 12, "THICKOUTLINE")
	name:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 2, -5)
	name:SetPoint("TOPRIGHT", self, "BOTTOMRIGHT", -2, -5)
	name:SetJustifyH("LEFT")
	self.Name = name


	local cb = CreateFrame("StatusBar", nil, self)
	cb:SetStatusBarTexture("Interface\\BUTTONS\\WHITE8X8")
	cb:SetPoint("TOP", self, "BOTTOM", 0, -5)
	cb:SetPoint("LEFT", 2+21, 0)
	cb:SetPoint("RIGHT", -2, 0)
	cb:SetHeight(19)

	cb.Icon = cb:CreateTexture(nil, "BORDER")
	cb.Icon:SetSize(19,19)
	cb.Icon:SetPoint("TOPRIGHT", cb, "TOPLEFT", -2, 0)
	cb.Icon:SetTexCoord(.1,.9,.1,.9)

	cb.bg = cb:CreateTexture(nil, "BACKGROUND")
	cb.bg:SetTexture(0,0,0)
	cb.bg:SetPoint("TOPLEFT", cb.Icon, "TOPLEFT", -2, 2)
	cb.bg:SetPoint("BOTTOMRIGHT", 2, -2)

	cb.Time = cb:CreateFontString(nil, "OVERLAY")
	cb.Time:SetFont(mediapath.."kimberley.ttf", 12, "THICKOUTLINE")
	cb.Time:SetPoint("RIGHT", cb, "RIGHT", -2, -1)
	cb.Time:SetJustifyH("RIGHT")

	cb.Text = cb:CreateFontString(nil, "OVERLAY")
	cb.Text:SetFont(mediapath.."kimberley.ttf", 12, "THICKOUTLINE")
	cb.Text:SetPoint("LEFT", cb, "LEFT", 2, -1)
	cb.Text:SetPoint("RIGHT", cb, "RIGHT", -50, 0)
	cb.Text:SetJustifyH("LEFT")

	cb.CustomTimeText = CustomCastbarTimeText
	self.Castbar = cb


	self.RaidIcon = self:CreateTexture(nil, "OVERLAY")
	self.RaidIcon:SetWidth(32)
	self.RaidIcon:SetHeight(32)
	self.RaidIcon:SetPoint("RIGHT", hp.bg, "LEFT", -4, 0)


	if unit~="player" then
		local auras = CreateFrame("Frame", nil, self)
		auras.size = 26
		auras.numBuffs = 40
		auras.numDebuffs = 40
		auras.initialAnchor = "BOTTOMLEFT"
		auras["growth-x"] = "RIGHT"
		auras["growth-y"] = "UP"
		auras:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, 3)
		auras.showDebuffType = false
		auras.gap = 1
		auras:SetHeight(auras.size)
		auras:SetWidth(256)
		auras.spacing = 1
		auras.PostCreateIcon = PostCreateIcon
		auras.PostUpdateIcon = PostUpdateIcon
		auras.PreUpdate = UnitAuraFilterChange
		auras.SetPosition = SetPosition
		self.Auras = auras
	end


	if unit == "player" then
		self.CPoints = {}
		for i = 1, 5 do
			self.CPoints[i] = self:CreateTexture(nil, "BACKGROUND")
			self.CPoints[i]:SetHeight(32)
			self.CPoints[i]:SetWidth(32)
			self.CPoints[i]:SetTexture(mediapath.."combopoint"..i)
			if(i==1) then
				self.CPoints[i]:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, 5)
			else
				self.CPoints[i]:SetPoint("LEFT", self.CPoints[i-1], "RIGHT", 0, 0)
			end
		end

		if PlayerClass=="DEATHKNIGHT" then
			self.Runes = CreateFrame("Frame", nil, self)
			self.Runes:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, 10)
			self.Runes:SetHeight(8)
			self.Runes:SetWidth(8)
			for i = 1, 6 do
				self.Runes[i] = CreateFrame("StatusBar", nil, self)
				self.Runes[i]:SetWidth(20)
				self.Runes[i]:SetHeight(8)
				if i==1 then
					self.Runes[i]:SetPoint("TOPLEFT", self.Runes, "TOPLEFT", 2, 0)
				else
					self.Runes[i]:SetPoint("LEFT", self.Runes[i-1], "RIGHT", 2, 0)
				end
				self.Runes[i]:SetStatusBarTexture("Interface\\\BUTTONS\\WHITE8X8")
				self.Runes[i].bg2 = self.Runes[i]:CreateTexture(nil, "BACKGROUND")
				self.Runes[i].bg2:SetPoint("TOPLEFT", -2, 2)
				self.Runes[i].bg2:SetPoint("BOTTOMRIGHT", 2, -2)
				self.Runes[i].bg2:SetTexture("Interface\\\BUTTONS\\WHITE8X8")
				self.Runes[i].bg2:SetVertexColor(0,0,0)
			end
		end

	end

end

oUF:RegisterStyle("StyleSolo", StyleFunc)
oUF:SetActiveStyle("StyleSolo")

oUF:Spawn("player"):SetPoint("CENTER", -200, -300)
oUF:Spawn("target"):SetPoint("CENTER", 200, -300)
oUF:Spawn("targettarget"):SetPoint("CENTER", 500, -300)
